#include <fintrf.h>
!
!     Matlab call :  [time_watched]=bargaining_alc_time_watched_mex(pre_gamma_small_market,pre_gamma_big_market,consumer_buys_small_market,consumer_buys_big_market,market_share_small_market,market_share_big_market,regulation_option,market_size)
!

!       pre_gamma_small_market                      simulations_small_market x number_of_channels+1
!       pre_gamma_big_market                        simulations_big_market x number_of_channels+1
!       consumer_buys_small_market                  simulations_small_market x number_of_channels x 3
!       consumer_buys_big_market                    simulations_big_market x number_of_channels x 3
!       market_share_small_market                   simulations_small_market x 3
!       market_share_big_market                     simulations_big_market x 3
!       regulation_option                           scalar
!       market_size                                 2 x 1
!       bundles                                     number_of_channels x 4
!       channel_in_question                         scalar

!       Output
!       time_watched                                number_of_channels x 1

!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs, channels_big_dimensions(3), channels_small_dimensions(3)
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer pre_gamma_small_market, pre_gamma_big_market, market_share_big_market, market_share_small_market, consumer_buys_big_market, consumer_buys_small_market, regulation_option, market_size
      mwpointer time_watched, bundles, channel_in_question
      mwsize mxgetm, mxgetn
      mwsize simulations_big_market, simulations_small_market, number_of_channels
    
!     RHS parameters
      
        pre_gamma_small_market=mxgetpr(prhs(1)) 
        pre_gamma_big_market =  mxgetpr(prhs(2)) 
        consumer_buys_small_market =  mxgetpr(prhs(3)) 
        consumer_buys_big_market =  mxgetpr(prhs(4)) 
        market_share_small_market =  mxgetpr(prhs(5)) 
        market_share_big_market = mxgetpr(prhs(6)) 
        regulation_option=mxgetpr(prhs(7))
        market_size=mxgetpr(prhs(8))
        bundles=mxgetpr(prhs(9))
        channel_in_question=mxgetpr(prhs(10))

        simulations_big_market=mxgetm(prhs(2))
        simulations_small_market=mxgetm(prhs(1))
        number_of_channels=mxgetn(prhs(2))-1

      PLHS(1) = mxcreatedoublematrix(number_of_channels,1,0)
      time_watched = mxgetpr(PLHS(1))


      call bargaining_alc_time_watched_channel_mex(%val(time_watched),%val(pre_gamma_small_market),%val(pre_gamma_big_market),%val(consumer_buys_small_market),%val(consumer_buys_big_market),%val(market_share_small_market),%val(market_share_big_market),%val(regulation_option),%val(market_size),%val(bundles),%val(channel_in_question),number_of_channels,simulations_big_market,simulations_small_market)
                   
      return
      end


        subroutine bargaining_alc_time_watched_channel_mex(time_watched,pre_gamma_small_market,pre_gamma_big_market,consumer_buys_small_market,consumer_buys_big_market,market_share_small_market,market_share_big_market,regulation_option,market_size,bundles,channel_in_question,number_of_channels,simulations_big_market,simulations_small_market)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations_big_market, simulations_small_market, number_of_channels
        integer i, j, k, m, n
        double precision time_watched(number_of_channels)
        double precision pre_gamma_small_market(simulations_small_market,number_of_channels+1), pre_gamma_big_market(simulations_big_market,number_of_channels+1)
        double precision market_share_big_market(simulations_big_market,3,2), market_share_small_market(simulations_small_market,3,2)
        double precision market_size(2), regulation_option
        double precision consumer_buys_big_market(simulations_big_market,number_of_channels,3), consumer_buys_small_market(simulations_small_market,number_of_channels,3)
        double precision bundle_gamma_big_market(simulations_big_market,number_of_channels,3,2), bundle_gamma_small_market(simulations_big_market,number_of_channels,3,2)
        double precision bundle_total_gamma_big_market(simulations_big_market,3,2), bundle_total_gamma_small_market(simulations_small_market,3,2)
        double precision time_watched_big_market(simulations_big_market,number_of_channels), time_watched_small_market(simulations_small_market,number_of_channels)
        double precision channel_in_question, bundles(number_of_channels,4)
        integer i_channel
        
        i_channel=INT(channel_in_question)

            do j=1,simulations_big_market
            
            do k=1,number_of_channels
            
            if (k.ne.i_channel) then
            bundle_gamma_big_market(j,k,1,1)=pre_gamma_big_market(j,k)*consumer_buys_big_market(j,k,1)*bundles(k,1)
            bundle_gamma_big_market(j,k,1,2)=pre_gamma_big_market(j,k)*consumer_buys_big_market(j,k,1)*bundles(k,1)
            bundle_gamma_big_market(j,k,2,1)=pre_gamma_big_market(j,k)*consumer_buys_big_market(j,k,2)*bundles(k,3)
            bundle_gamma_big_market(j,k,2,2)=pre_gamma_big_market(j,k)*consumer_buys_big_market(j,k,2)*bundles(k,3)
            bundle_gamma_big_market(j,k,3,1)=pre_gamma_big_market(j,k)*consumer_buys_big_market(j,k,3)*bundles(k,4)
            bundle_gamma_big_market(j,k,3,2)=pre_gamma_big_market(j,k)*consumer_buys_big_market(j,k,3)*bundles(k,4)
            elseif (k.eq.i_channel) then
            bundle_gamma_big_market(j,k,1,1)=0d0
            bundle_gamma_big_market(j,k,1,2)=pre_gamma_big_market(j,k)*bundles(k,1)
            bundle_gamma_big_market(j,k,2,1)=0d0
            bundle_gamma_big_market(j,k,2,2)=pre_gamma_big_market(j,k)*bundles(k,3)
            bundle_gamma_big_market(j,k,3,1)=0d0
            bundle_gamma_big_market(j,k,3,2)=pre_gamma_big_market(j,k)*bundles(k,4)

            end if

            
            end do

            bundle_total_gamma_big_market(j,1,1)=SUM(bundle_gamma_big_market(j,:,1,1))+pre_gamma_big_market(j,number_of_channels+1)
            bundle_total_gamma_big_market(j,1,2)=SUM(bundle_gamma_big_market(j,:,1,2))+pre_gamma_big_market(j,number_of_channels+1)
            bundle_total_gamma_big_market(j,2,1)=SUM(bundle_gamma_big_market(j,:,2,1))+pre_gamma_big_market(j,number_of_channels+1)
            bundle_total_gamma_big_market(j,2,2)=SUM(bundle_gamma_big_market(j,:,2,2))+pre_gamma_big_market(j,number_of_channels+1)
            bundle_total_gamma_big_market(j,3,1)=SUM(bundle_gamma_big_market(j,:,3,1))+pre_gamma_big_market(j,number_of_channels+1)
            bundle_total_gamma_big_market(j,3,2)=SUM(bundle_gamma_big_market(j,:,3,2))+pre_gamma_big_market(j,number_of_channels+1)

            do k=1,number_of_channels
            time_watched_big_market(j,k)= (bundle_gamma_big_market(j,k,1,1)/bundle_total_gamma_big_market(j,1,1))*market_share_big_market(j,1,1)+(bundle_gamma_big_market(j,k,2,1)/bundle_total_gamma_big_market(j,2,1))*market_share_big_market(j,2,1)+(bundle_gamma_big_market(j,k,3,1)/bundle_total_gamma_big_market(j,3,1))*market_share_big_market(j,3,1) + (bundle_gamma_big_market(j,k,1,2)/bundle_total_gamma_big_market(j,1,2))*market_share_big_market(j,1,2)+(bundle_gamma_big_market(j,k,2,2)/bundle_total_gamma_big_market(j,2,2))*market_share_big_market(j,2,2)+(bundle_gamma_big_market(j,k,3,2)/bundle_total_gamma_big_market(j,3,2))*market_share_big_market(j,3,2)
            end do

            end do


            do j=1,simulations_small_market
            
            do k=1,number_of_channels
            
            if (k.ne.i_channel) then
            bundle_gamma_small_market(j,k,1,1)=pre_gamma_small_market(j,k)*consumer_buys_small_market(j,k,1)*bundles(k,2)
            bundle_gamma_small_market(j,k,1,2)=pre_gamma_small_market(j,k)*consumer_buys_small_market(j,k,1)*bundles(k,2)
            bundle_gamma_small_market(j,k,2,1)=pre_gamma_small_market(j,k)*consumer_buys_small_market(j,k,2)*bundles(k,3)
            bundle_gamma_small_market(j,k,2,2)=pre_gamma_small_market(j,k)*consumer_buys_small_market(j,k,2)*bundles(k,3)
            bundle_gamma_small_market(j,k,3,1)=pre_gamma_small_market(j,k)*consumer_buys_small_market(j,k,3)*bundles(k,4)
            bundle_gamma_small_market(j,k,3,2)=pre_gamma_small_market(j,k)*consumer_buys_small_market(j,k,3)*bundles(k,4)
            elseif (k.eq.i_channel) then
            bundle_gamma_small_market(j,k,1,1)=0d0
            bundle_gamma_small_market(j,k,1,2)=pre_gamma_small_market(j,k)*bundles(k,2)
            bundle_gamma_small_market(j,k,2,1)=0d0
            bundle_gamma_small_market(j,k,2,2)=pre_gamma_small_market(j,k)*bundles(k,3)
            bundle_gamma_small_market(j,k,3,1)=0d0
            bundle_gamma_small_market(j,k,3,2)=pre_gamma_small_market(j,k)*bundles(k,4)

            end if

            
            end do

            bundle_total_gamma_small_market(j,1,1)=SUM(bundle_gamma_small_market(j,:,1,1))+pre_gamma_small_market(j,number_of_channels+1)
            bundle_total_gamma_small_market(j,1,2)=SUM(bundle_gamma_small_market(j,:,1,2))+pre_gamma_small_market(j,number_of_channels+1)
            bundle_total_gamma_small_market(j,2,1)=SUM(bundle_gamma_small_market(j,:,2,1))+pre_gamma_small_market(j,number_of_channels+1)
            bundle_total_gamma_small_market(j,2,2)=SUM(bundle_gamma_small_market(j,:,2,2))+pre_gamma_small_market(j,number_of_channels+1)
            bundle_total_gamma_small_market(j,3,1)=SUM(bundle_gamma_small_market(j,:,3,1))+pre_gamma_small_market(j,number_of_channels+1)
            bundle_total_gamma_small_market(j,3,2)=SUM(bundle_gamma_small_market(j,:,3,2))+pre_gamma_small_market(j,number_of_channels+1)

            do k=1,number_of_channels
            time_watched_small_market(j,k)= (bundle_gamma_small_market(j,k,1,1)/bundle_total_gamma_small_market(j,1,1))*market_share_small_market(j,1,1)+(bundle_gamma_small_market(j,k,2,1)/bundle_total_gamma_small_market(j,2,1))*market_share_small_market(j,2,1)+(bundle_gamma_small_market(j,k,3,1)/bundle_total_gamma_small_market(j,3,1))*market_share_small_market(j,3,1) + (bundle_gamma_small_market(j,k,1,2)/bundle_total_gamma_small_market(j,1,2))*market_share_small_market(j,1,2)+(bundle_gamma_small_market(j,k,2,2)/bundle_total_gamma_small_market(j,2,2))*market_share_small_market(j,2,2)+(bundle_gamma_small_market(j,k,3,2)/bundle_total_gamma_small_market(j,3,2))*market_share_small_market(j,3,2)
            end do

            end do


        
        do k=1,number_of_channels
            time_watched(k) = (market_size(2)/SUM(market_size))*SUM(time_watched_small_market(:,k))/simulations_small_market + (market_size(1)/SUM(market_size))*SUM(time_watched_big_market(:,k))/simulations_big_market
          end do

        return
        end

