#include <fintrf.h>
!
!     Matlab call :  [player_profits profits market_share_big_market market_share_small_market]=bargaining_downstream_firm_profits_mex(player,p1,p2,p3,p4,bundles,beta_i_big_market,channels_utility_big_market,beta_i_small_market,channels_utility_small_market,input_cost,logit_var,regulation_option,market_size,compute_upstream_revenue,utility_constants);
!
!       player                                      scalar
!       p1                                          scalar
!       p2                                          scalar
!       p3                                          scalar
!       p4                                          scalar
!       bundles                                     number_of_channels x 4
!       beta_i_big_market                           simulations_big_market x 1
!       channels_utility_big_market                 number_of_channels x simulations_big_market
!       beta_i_small_market                         simulations_small_market x 1
!       channels_utility_small_market               number_of_channels x simulations_small_market
!       input_cost                                  number_of_channels x 4
!       logit_var                                   scalar
!       regulation_option                           scalar
!       market_size                                 2 x 1
!       compute_upstream_revenue                    scalar
!       utility_constants                           4 x 1
!       outside_good_utility                        simulations_big_market x 1


!       Output
!      	player_profits                              scalar
!      	profits                                     4 x 1
!       market_share_big_market                     simulations_big_market x 3
!       market_share_small_market                   simulations_small_market x 3
!       upstream_fee_revenue                        number_of_channels x 1


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer player, p1, p2, p3, p4, beta_i_big_market, channels_utility_big_market, beta_i_small_market, channels_utility_small_market, input_cost, logit_var, regulation_option, market_size, bundles, outside_good_utility
      mwpointer profits, player_profits, market_share_big_market, market_share_small_market, compute_upstream_revenue, upstream_fee_revenue, utility_constants
      mwsize mxgetm, mxgetn
      mwsize simulations_big_market, simulations_small_market, number_of_channels
    
!     RHS parameters
      
        player=mxgetpr(prhs(1)) 
        p1 =  mxgetpr(prhs(2)) 
        p2 =  mxgetpr(prhs(3)) 
        p3 =  mxgetpr(prhs(4)) 
        p4 =  mxgetpr(prhs(5)) 
        bundles = mxgetpr(prhs(6)) 
        beta_i_big_market = mxgetpr(prhs(7)) 
        channels_utility_big_market = mxgetpr(prhs(8)) 
        beta_i_small_market = mxgetpr(prhs(9)) 
        channels_utility_small_market = mxgetpr(prhs(10)) 
        input_cost = mxgetpr(prhs(11))
        logit_var=mxgetpr(prhs(12)) 
        regulation_option=mxgetpr(prhs(13))
        market_size=mxgetpr(prhs(14))
        compute_upstream_revenue=mxgetpr(prhs(15))
        utility_constants=mxgetpr(prhs(16))
        outside_good_utility=mxgetpr(prhs(17))

        simulations_big_market=mxgetm(prhs(7))
        simulations_small_market=mxgetm(prhs(9))
        number_of_channels=mxgetm(prhs(11))

      PLHS(1) = mxcreatedoublescalar(0)
      player_profits = mxgetpr(PLHS(1))
      PLHS(2) = mxcreatedoublematrix(4,1,0)
      profits = mxgetpr(PLHS(2))
      PLHS(3) = mxcreatedoublematrix(simulations_big_market,3,0)
      market_share_big_market = mxgetpr(PLHS(3))
      PLHS(4) = mxcreatedoublematrix(simulations_small_market,3,0)
      market_share_small_market = mxgetpr(PLHS(4))
      PLHS(5) = mxcreatedoublematrix(number_of_channels,1,0)
      upstream_fee_revenue = mxgetpr(PLHS(5))

      call bargaining_downstream_firm_profits_mex(%val(player_profits),%val(profits),%val(market_share_big_market),%val(market_share_small_market),%val(upstream_fee_revenue),%val(player),%val(p1),%val(p2),%val(p3),%val(p4),%val(bundles),%val(beta_i_big_market),%val(channels_utility_big_market),%val(beta_i_small_market),%val(channels_utility_small_market),%val(input_cost),%val(logit_var),%val(regulation_option),%val(market_size),%val(compute_upstream_revenue),%val(utility_constants),%val(outside_good_utility),number_of_channels,simulations_big_market,simulations_small_market)
                   
      return
      end


        subroutine bargaining_downstream_firm_profits_mex(player_profits,profits,market_share_big_market,market_share_small_market,upstream_fee_revenue,player,p1,p2,p3,p4,bundles,beta_i_big_market,channels_utility_big_market,beta_i_small_market,channels_utility_small_market,input_cost,logit_var,regulation_option,market_size,compute_upstream_revenue,utility_constants,outside_good_utility,number_of_channels,simulations_big_market,simulations_small_market)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations_big_market, simulations_small_market, number_of_channels
        integer i, j, k, m, n, i_player 
        double precision profits(4), player, bundles(number_of_channels,4), p1, p2, p3, p4, input_cost(number_of_channels,4), logit_var, regulation_option
        double precision consumer_utility(simulations_big_market,3), consumer_utility_small(simulations_small_market,3), mu_eps, denominator(simulations_big_market), denominator_small(simulations_small_market), total_share(4)
        double precision beta_i_big_market(simulations_big_market), channels_utility_big_market(number_of_channels,simulations_big_market), beta_i_small_market(simulations_small_market), channels_utility_small_market(number_of_channels,simulations_small_market)
        double precision bundle_utility_big_market(simulations_big_market), bundle_utility_small_market(simulations_small_market), market_share_big_market(simulations_big_market,3), market_share_small_market(simulations_small_market,3)
        double precision player_profits, market_size(2), compute_upstream_revenue, upstream_fee_revenue(number_of_channels), utility_constants(4), outside_good_utility(simulations_big_market)

        mu_eps=1d0/logit_var

        i_player=INT(player)


if (regulation_option.eq.1d0) then
!Bundling

!-!$OMP PARALLEL SHARED(channels_utility_big_market,beta_i_big_market,channels_utility_small_market,beta_i_small_market,bundles,p1,p2,p3,p4,consumer_utility,denominator,market_share_big_market,market_share_small_market,mu_eps,outside_good_utility), NUM_THREADS(12)

!-!$OMP DO SCHEDULE(DYNAMIC,100)
            
               
            do j=1,simulations_big_market

            consumer_utility(j,1) = EXP( (utility_constants(1) + DOT_PRODUCT(channels_utility_big_market(:,j),bundles(:,1)) + beta_i_big_market(j)*p1)*mu_eps)
            consumer_utility(j,2) = EXP( (utility_constants(3) + DOT_PRODUCT(channels_utility_big_market(:,j),bundles(:,3)) + beta_i_big_market(j)*p3)*mu_eps)
            consumer_utility(j,3) = EXP( (utility_constants(4) + DOT_PRODUCT(channels_utility_big_market(:,j),bundles(:,4)) + beta_i_big_market(j)*p4)*mu_eps)

            denominator(j) = outside_good_utility(j)+SUM(consumer_utility(j,:))

            market_share_big_market(j,1) = consumer_utility(j,1)/denominator(j)
            market_share_big_market(j,2) = consumer_utility(j,2)/denominator(j)
            market_share_big_market(j,3) = consumer_utility(j,3)/denominator(j)

            end do
    
!-!$OMP END DO NOWAIT

!-!$OMP DO SCHEDULE(DYNAMIC,100)

            do j=1,simulations_small_market

            consumer_utility_small(j,1) = EXP( (utility_constants(2) + DOT_PRODUCT(channels_utility_small_market(:,j),bundles(:,2)) + beta_i_small_market(j)*p2)*mu_eps)
            consumer_utility_small(j,2) = EXP( (utility_constants(3) + DOT_PRODUCT(channels_utility_small_market(:,j),bundles(:,3)) + beta_i_small_market(j)*p3)*mu_eps)
            consumer_utility_small(j,3) = EXP( (utility_constants(4) + DOT_PRODUCT(channels_utility_small_market(:,j),bundles(:,4)) + beta_i_small_market(j)*p4)*mu_eps)

            denominator_small(j) = outside_good_utility(j)+SUM(consumer_utility_small(j,:))

            market_share_small_market(j,1) = consumer_utility_small(j,1)/denominator_small(j)
            market_share_small_market(j,2) = consumer_utility_small(j,2)/denominator_small(j)
            market_share_small_market(j,3) = consumer_utility_small(j,3)/denominator_small(j)

            end do

!-!$OMP END DO NOWAIT

!-!$OMP END PARALLEL

            total_share(1) = SUM(market_share_big_market(:,1))*market_size(1)
            profits(1) = (p1-DOT_PRODUCT(input_cost(:,1),bundles(:,1)))*total_share(1)
            total_share(2) = SUM(market_share_small_market(:,1))*market_size(2)
            profits(2) = (p2-DOT_PRODUCT(input_cost(:,2),bundles(:,2)))*total_share(2)
            total_share(3) = SUM(market_share_big_market(:,2))*market_size(1) + SUM(market_share_small_market(:,2))*market_size(2)
            profits(3) = (p3-DOT_PRODUCT(input_cost(:,3),bundles(:,3)))*total_share(3)
            total_share(4) = SUM(market_share_big_market(:,3))*market_size(1) + SUM(market_share_small_market(:,3))*market_size(2)
            profits(4) = (p4-DOT_PRODUCT(input_cost(:,4),bundles(:,4)))*total_share(4)

            player_profits=-profits(i_player)

            if (compute_upstream_revenue.eq.1) then
            do k=1,number_of_channels
            upstream_fee_revenue(k) = bundles(k,1)*input_cost(k,1)*total_share(1) + bundles(k,2)*input_cost(k,2)*total_share(2) + bundles(k,3)*input_cost(k,3)*total_share(3) + bundles(k,4)*input_cost(k,4)*total_share(4)
            end do 
            end if

end if        

        return
        end

