% Bargaining Game Main Code

clear all

% Preliminaries

% Regulatory regime option
% 1=bundling
% 2=BSP
% 3=themed tiers
% 4=ALC
regulation_option=4;

% Estimation and/or Counterfactual 
estimation=0;
counterfactual=1;

% Conglomerate or Individual Channel Bargaining
conglomerate=1;

% Initial values
load all_parameters
demographics=csvread('data/demographics.csv');
[simulations cable_tolerance specification channels_to_add_drop parametric_family_of_marginal_distributions copula_parametric_family D_names]=cost_options(1);

number_of_channels=sum(specification(:,3));
% if (regulation_option==1)
bundle=[ones(1,number_of_channels) 1];
% else
alc_bundle=[zeros(1,number_of_channels) 1];
alc_bundle([1 2 3 4 7 9 10 12 13 16 17 18 19 20 25 26 27 34 38 39 40 41 45 46])=1;
% end



run input_cost_specification
load channel_ad_revenue

sims=1500;
wtp_distribution=zeros(sims,number_of_channels);
channels_utility=zeros(sims,number_of_channels);
pre_gamma=zeros(sims,number_of_channels);
input_cost=zeros(number_of_channels,4);
ad_coefficient=zeros(number_of_channels,1);

[mkt_demographics sample_unobservable_draws sample_zero_out_draws]=set_random_draws_single_market(1,mean(demographics),sims,number_of_channels+1);

k=1;
for j=1:length(specification)
    if (specification(j,3)==1)
        [wtp_distribution(:,k) channels_utility(:,k) beta_i mkt_demographics pre_gamma]= sample_wtp(all_parameters,k,bundle,mkt_demographics,sample_unobservable_draws,sample_zero_out_draws,specification,sims);
        [wtp_distribution_alc(:,k) channels_utility_alc(:,k) beta_i mkt_demographics pre_gamma]= sample_wtp(all_parameters,k,alc_bundle,mkt_demographics,sample_unobservable_draws,sample_zero_out_draws,specification,sims);
        input_cost(k,:)=[(-.03+1.04*raw_input_cost(j))*exp(-.0045*24) (-.03+1.04*raw_input_cost(j)) (-.03+1.04*raw_input_cost(j))*exp(-.0045*15) (-.03+1.04*raw_input_cost(j))*exp(-.0045*10)];
        ad_coefficient(k)=channel_ad_revenue(j);
        k=k+1;
    end
end
outside_good_utility = ones(sims,1);
wtp_distribution=max(wtp_distribution,0);

if (regulation_option>1)
    wtp_distribution=wtp_distribution_alc;
    channels_utility=channels_utility_alc;
end

if (conglomerate==1)
conglomerate_groups{1}=[1	14	16	17	35	42];						
conglomerate_groups{2}=[2	49	];
conglomerate_groups{3}=[3	13	26	40	43	];
conglomerate_groups{4}=[7	9	12	38	41	];					
conglomerate_groups{5}=[15	23	47			];					
conglomerate_groups{6}=[19	20	21	31	36	];					
conglomerate_groups{7}=[4	6	8	28	34	46];					
conglomerate_groups{8}=[5	10	11	22	29	30	32	37	45	48];
conglomerate_groups{9}=[18	25];
conglomerate_groups{10}=[24];										
conglomerate_groups{11}=[27];										
conglomerate_groups{12}=[33];										
conglomerate_groups{13}=[39];										
conglomerate_groups{14}=[44];										
channel_to_conglomerate=[1 2 3 7 8 7 4 7 4 8 8 4 3 1 5 1 1 9 6 6 6 8 5 10 9 3 11 7 8 8 6 8 12 7 1 6 8 4 13 3 4 1 3 14 8 7 5 8 2];
conglomerate_parameters={conglomerate,conglomerate_groups,channel_to_conglomerate};
else
    conglomerate_parameters={0,0,0};
end


beta_i_small_market=beta_i;
beta_i_big_market=beta_i;
channels_utility_small_market=channels_utility;
channels_utility_big_market=channels_utility;
pre_gamma_small_market=pre_gamma';
pre_gamma_big_market=pre_gamma';

market_size=[20 1];

logit_var=   4;
utility_constants=[8 8 0 0];

preferences{1}=beta_i_big_market;
preferences{2}=channels_utility_big_market;
preferences{3}=pre_gamma_big_market;
preferences{4}=beta_i_small_market;
preferences{5}=channels_utility_small_market;
preferences{6}=pre_gamma_small_market;
preferences{7}=logit_var;
preferences{8}=market_size;
preferences{9}=utility_constants;
preferences{10}=ad_coefficient;
preferences{11}=outside_good_utility;

if (regulation_option==1)
last_prices={40, 40, 40, 40};
elseif (regulation_option==2)
    last_prices={[30 40], [30 40], [30 40], [30 40]};
elseif (regulation_option==3)
    run bargaining_tt_preliminary
    last_prices={p1,p2,p3,p4}; 
    save last_prices_tt last_prices
elseif (regulation_option==4)
    p_trial=[input_cost(:,1); 20];
    last_prices={p_trial, p_trial, p_trial, p_trial};
end
bundles=ones(size(input_cost));

%Calibrate utility constants and logit var
% prob = clsAssign(@(z) bargaining_set_basic_parameters(z,last_prices,bundles,input_cost,regulation_option,preferences), [],[],  [-Inf -Inf -Inf -Inf 1], [Inf Inf Inf Inf Inf], 'NLLS', [utility_constants logit_var], ones(5,1), [],[],[],[],[],[],[],[],[],[],[],[],[]);
% prob.KNITRO.options.XTOL=1e-8;
% prob.KNITRO.options.ALG = 3;
% prob.KNITRO.options.BAR_FEASIBLE = 1;
% prob.KNITRO.options.BAR_FEASMODETOL = 1e-15;
% prob.KNITRO.options.HONORBNDS = 1;
% prob.PriLevelOpt = 4; % Print one line each iteration
% prob.KNITRO.options.HESSOPT = 3;
% res = tomRun('knitro', prob, 2);
% intermediate=res.x_k;
% % save intermediate intermediate

load intermediate
utility_constants=intermediate(1:4)';
logit_var=  intermediate(end);
preferences{7}=logit_var;
preferences{9}=utility_constants;


load current_estimated_bargaining_parameters
% estimated_bargaining_parameters=.5*ones(size(input_cost));
bargaining_parameters=estimated_bargaining_parameters;


new_input_costs=input_cost;

% save estimated_bargaining_parameters
% 
if (estimation==1)
% If estimation, call minimizer function


knitro=0;
profile on
        marked_groups=zeros(14,1);
for k=1:length(input_cost)

    
        
        optimization_options=optimset('TolX',5e-3,'TolFun',1e-3);

        if (conglomerate==0)
            
            est_tolerance=1;
            while (est_tolerance>1e-3);     
            old_estimated_bargaining_parameters=estimated_bargaining_parameters;
            for yyy=1:4
            estimated_bargaining_parameters(k,yyy)=fminsearch(@(z) uni_bargaining_estimation_objective_function(z,k,yyy,input_cost,estimated_bargaining_parameters,regulation_option,preferences,conglomerate_parameters,knitro),estimated_bargaining_parameters(k,yyy),optimization_options);
            save estimated_bargaining_parameters
            end
            est_tolerance=max(max(abs(old_estimated_bargaining_parameters-estimated_bargaining_parameters)));
            end
        else

            if (marked_groups(channel_to_conglomerate(k))==0)
            est_tolerance=1;
            while (est_tolerance>1e-3);
                old_estimated_bargaining_parameters=estimated_bargaining_parameters;
                for yyy=1:4    
                estimated_bargaining_parameters(k,yyy)=fminsearch(@(z) uni_bargaining_estimation_objective_function(z,k,yyy,input_cost,estimated_bargaining_parameters,regulation_option,preferences,conglomerate_parameters,knitro),estimated_bargaining_parameters(k,yyy),optimization_options);
%                 
%                     prob = conAssign(@(z) uni_bargaining_estimation_objective_function(z,k,yyy,input_cost,estimated_bargaining_parameters,regulation_option,preferences,conglomerate_parameters,knitro), [],[],[],0,1,'NP', estimated_bargaining_parameters(k,yyy));
%                     prob.KNITRO.options.XTOL=5e-3;
%                     prob.KNITRO.options.ALG = 1;
%                     prob.PriLevOpt = 2; 
%                     res = tomRun('knitro', prob);
                
                save estimated_bargaining_parameters
                end
                est_tolerance=max(max(abs(old_estimated_bargaining_parameters-estimated_bargaining_parameters)));
                estimated_bargaining_parameters(conglomerate_groups{channel_to_conglomerate(k)},:)=repmat(estimated_bargaining_parameters(k,:),length(conglomerate_groups{channel_to_conglomerate(k)}),1);
            end
            marked_groups(channel_to_conglomerate(k))=1;
            end
%   estimated_bargaining_parameters(k,:)=fminsearch(@(z) bargaining_estimation_objective_function(z,k,input_cost,bargaining_parameters(k,:),regulation_option,preferences,conglomerate_parameters,knitro),estimated_bargaining_parameters(k,1)*ones(1,4),optimization_options);
%         estimated_bargaining_parameters(conglomerate_groups{channel_to_conglomerate(k)},:)=repmat(estimated_bargaining_parameters(k,:),length(conglomerate_groups{channel_to_conglomerate(k)}),1);
        end
        
        save estimated_bargaining_parameters
        
end

elseif (counterfactual==1)


if (conglomerate==0)
    
     
        counterfactual_tolerance=1;
        while (counterfactual_tolerance>.05)
        old_input_costs=new_input_costs;
        for k=1:length(input_cost(:,1))
        new_input_costs=bargaining_equilibrium(k,new_input_costs,bundles,bargaining_parameters,regulation_option,preferences,conglomerate_parameters);
        end
        counterfactual_tolerance=max(max(abs(new_input_costs-old_input_costs)));
        disp(counterfactual_tolerance)
        end
            
    
elseif (conglomerate==1)
        
        counterfactual_tolerance=1;
        while (counterfactual_tolerance>.05)
        old_input_costs=new_input_costs;
            
        for k=1:length(conglomerate_groups)
        new_input_costs=bargaining_equilibrium(conglomerate_groups{k}(1),new_input_costs,bundles,bargaining_parameters,regulation_option,preferences,conglomerate_parameters);
        save track_cf_ten_percent_channel_markup
        end
        
        counterfactual_tolerance=max(max(abs(new_input_costs-old_input_costs)));
        disp(counterfactual_tolerance)
        end
    
end
        

end






