%% Data Import for Model Estimation
%% Crawford/Yurukoglu
%% The Welfare Effects of Bundling in Multichannel Television


%% IMPORT BUNDLE AND MARKET DATA


% Every vector has been exported in the following order:
% Year: 1997-2007
% DMA: 1-210
% Market ID: 1-max(d,y)
% Bundle ID: 1-max(d,y,m)

% [Year  DMA  DMA_Market_ID  DMA_Market_Count  DMA_BundleID  DMA_Bundle_Count
% Bundle_Count Bundle_ID]
dma_order=csvread('data/dma_market_and_bundle_id.csv');

regional_sports_networks=[2 6 59:65 70 99 146:151 153:169 223 224 248 306:308 320 340 374];

% These are all the same length as dma_order. In each of the following,
% each row has the relevant data for each bundle. For example, each row in
% channels has dummy variables for the bundle in the corresponding row in
% dma_order.
channels=csvread('data/channels.csv');
channels(:,146)=max(channels(:,regional_sports_networks)')';
bundle_dummies=csvread('data/bundle_dummies.csv');
bundle_prices=csvread('data/prices.csv');
dma_avg_prices=csvread('data/dma_avg_prices.csv');

bundle_market_shares=csvread('data/mshare.csv');
homespassed=csvread('data/homespassed.csv');
system_relative_dma_size=csvread('data/relative_size.csv');
dma_dummies=csvread('data/dma_dummies.csv');

more_weight=csvread('data/more_weight_in_viewership.csv');
pre_normalize_more_weight = (more_weight.*50 + (more_weight==0)) ;
normalize_more_weight= pre_normalize_more_weight./sum(pre_normalize_more_weight);

demographics=csvread('data/demographics.csv');
% demographics=demographics_read(:,3:end);
% clear demographics_read
% 1)year	2)dma	3)demog_family_pct 4)demog_family_3_or_more_pct	5)demog_household_income_0_10	6)demog_household_income_10_25	7)demog_household_income_25_40	8)demog_household_income_40_60	9)demog_household_income_60_75	10)demog_household_income_75_100	11)demog_household_income_100_150	12)demog_household_income_150_plus	13)demog_urban_pct	14)demog_rural_pct	15)demog_white_pct	16)demog_black_pct	17)demog_asian_pct	18)demog_hispanic_pct	19)demog_bachelors_degree_pct	20)demog_higher_degree_pct	21)demog_age_0_12	22)demog_age_13_18	23)demog_age_18_29	24)demog_age_30_39	25)demog_age_40_59



number_of_channels=sum(specification(:,3));
% Demand RHS X
% Narrows data into what the specification calls for.
X = zeros(length(bundle_prices(:,1)),number_of_channels);
X(:,1)=ones(length(bundle_prices(:,1)),1);
k=2;
for j=1:length(channels(1,:))
    if (specification(j,1)==1)
        X(:,k)=channels(:,j);
        k=k+1;
    end
end

homespassed_share=csvread('data/homes_passed_share.csv');
homespassed_share=((homespassed_share>0).*normalize_more_weight)./sum(((homespassed_share>0).*normalize_more_weight));
homespassed_share_new=homespassed_share;

homespassed_matrix_deltas=homespassed_share*ones(1,simulations_deltas);
homespassed_matrix_gammas=homespassed_share*ones(1,simulations_gammas);


l=1;
for j=1:length(bundle_dummies(1,:))
    if (sum(bundle_dummies(:,j))/20000)>.001
        bundle_dummies_new(:,l)=bundle_dummies(:,j); %#ok<AGROW>
        l=l+1;
      track(j)=1;
     end
end

X_small=[ones(length(bundle_dummies_new(:,1)),1) bundle_dummies_new bundle_prices];
% X_small=[X bundle_dummies_new bundle_prices];
Z_hausman=[X_small(:,1:end-1) dma_avg_prices];
chan_indices=find(mean(X)>.1);
Z_channels=[X_small(:,1:end-1) X(:,chan_indices(2:end))]; 
Z_both=[X_small(:,1:end-1) X(:,2:end) dma_avg_prices]; 

X_small_hat_hausman=Z_hausman*((Z_hausman'*Z_hausman)\(Z_hausman'*X_small));
X_small_hat_channels=Z_channels*((Z_channels'*Z_channels)\(Z_channels'*X_small));
X_small_hat_both=Z_both*((Z_both'*Z_both)\(Z_both'*X_small));

iv_choice=1;
if (iv_choice==1)
    Z=Z_hausman;
    X_small_hat=X_small_hat_hausman;
elseif (iv_choice==2)
    Z=Z_channels;
    X_small_hat=X_small_hat_channels;    
elseif (iv_choice==3)
    Z=Z_both;
    X_small_hat=X_small_hat_both;    
end
    
% Supply Side

Q=[X bundle_dummies_new];   

%% END BUNDLE AND MARKET DATA


%% SATELLITE DATA

% Satellite penetration rates by DMA/Year
satellite_penetration=zeros(11,210);
outsidegood_dmashare=zeros(11,210);
z=csvread('data/satellite_penetration_1997.csv');
satellite_penetration(1,:)=z(:,2);
outsidegood_dmashare(1,:)=z(:,3);
z=csvread('data/satellite_penetration_1998.csv');
satellite_penetration(2,:)=z(:,2);
outsidegood_dmashare(2,:)=z(:,3);
z=csvread('data/satellite_penetration_1999.csv');
satellite_penetration(3,:)=z(:,2);
outsidegood_dmashare(3,:)=z(:,3);
z=csvread('data/satellite_penetration_2000.csv');
satellite_penetration(4,:)=z(:,2);
outsidegood_dmashare(4,:)=z(:,3);
z=csvread('data/satellite_penetration_2001.csv');
satellite_penetration(5,:)=z(:,2);
outsidegood_dmashare(5,:)=z(:,3);
z=csvread('data/satellite_penetration_2002.csv');
satellite_penetration(6,:)=z(:,2);
outsidegood_dmashare(6,:)=z(:,3);
z=csvread('data/satellite_penetration_2003.csv');
satellite_penetration(7,:)=z(:,2);
outsidegood_dmashare(7,:)=z(:,3);
z=csvread('data/satellite_penetration_2004.csv');
satellite_penetration(8,:)=z(:,2);
outsidegood_dmashare(8,:)=z(:,3);
z=csvread('data/satellite_penetration_2005.csv');
satellite_penetration(9,:)=z(:,2);
outsidegood_dmashare(9,:)=z(:,3);
z=csvread('data/satellite_penetration_2006.csv');
satellite_penetration(10,:)=z(:,2);
outsidegood_dmashare(10,:)=z(:,3);
z=csvread('data/satellite_penetration_2007.csv');
satellite_penetration(11,:)=z(:,2);
outsidegood_dmashare(11,:)=z(:,3);
clear z
satellite_share=zeros(length(dma_order(:,1)),1);
outside_share=zeros(length(dma_order(:,1)),1);

for i=1:length(dma_order(:,1))

    for y=1:11
        for d=1:210
            
        if (dma_order(i,1)==1996+y && dma_order(i,2)==d)
            
            satellite_share(i)=satellite_penetration(y,d);
            outside_share(i)=outsidegood_dmashare(y,d);
            
        end
        
        end
    end
    
end

satellite_raw=csvread('data/directv.csv');
satellite_channels_raw=satellite_raw(:,3:end);
satellite_channels_raw(:,146)=1;
satellite_prices=satellite_raw(:,2);
satellite_year=satellite_raw(:,1);

satellite_channels = zeros(length(satellite_channels_raw(:,1)),number_of_channels);

k=1;
for j=1:length(channels(1,:))
    if (specification(j,1)==1)
        satellite_channels(:,k)=satellite_channels_raw(:,j);
        k=k+1;
    end
end

satellite_channels=ones(size(satellite_channels));

dma_size_weights=csvread('data/dma_size_weights.csv');

satellite_characteristics=[satellite_channels satellite_prices];

save satellite_data satellite_share outside_share satellite_channels satellite_prices dma_size_weights

%% END SATELLITE DATA



%% IMPORT VIEWERSHIP DATA (Nielsen then Mediamark)
% Creates P_chans and Sigma matrices by multivariate regression. This is
% the first stage estimation described in the paper in section 5.1.


% Import files
nielsen_raw_data=csvread('data/nielsen_ratings.csv');
nielsen_raw_data(:,3+146)=max(nielsen_raw_data(:,3+regional_sports_networks)')';
raw_cablerating=csvread('data/nielsen_agg_cablerating.csv'); %#ok<NASGU>
raw_cablecume=csvread('data/nielsen_agg_cablecume.csv'); %#ok<NASGU>

raw_coveragearearating=csvread('data/nielsen_agg_coveragearearating.csv'); %#ok<NASGU>
raw_coverageareacume=csvread('data/nielsen_agg_coverageareacume.csv');

raw_universerating=csvread('data/nielsen_agg_universerating.csv'); %#ok<NASGU>
raw_universecume=csvread('data/nielsen_agg_universecume.csv');

raw_mediamark=csvread('data/mediamark_full.csv');
raw_mediamark(:,2+146)=max(raw_mediamark(:,2+regional_sports_networks)')';

k=1;
for j=1:375
    if (specification(j,3)==1)
        % Fraction of household/hours tuned into channel
        nielsen_dma_ratings(:,k)=nielsen_raw_data(:,3+j);
        universe_cume(:,k)=raw_universecume(:,1+j);
        coverage_cume(:,k)=raw_coverageareacume(:,1+j);
        mediamark_ratings(:,k)=raw_mediamark(:,2+j);
        k=k+1;
    end
end

% mediamark_identifiers=raw_mediamark(:,1:5); % [year zipcode groupid1 groupid2 groupid3]
mediamark_identifiers=raw_mediamark(:,1:2); % [id wave]
mediamark_demographics=raw_mediamark(:,378:386); % [hhsize hhincome white black hispanic asian bachelors post_grad age] 
mediamark_subscriptions=raw_mediamark(:,387:end); % [cable digital_cable directv dish]
nielsen_demographics=nielsen_raw_data(:,379:386); % [urban family income black hispanic asian college age]
nielsen_dmas_years=nielsen_raw_data(:,[1 387]);
clear nielsen_raw_data raw_cablerating raw_cablecume raw_coveragearearating raw_coverageareacume raw_universerating raw_universecume

actual_dma_groups = unique(nielsen_dmas_years,'rows');
predicted_dma_groups=dma_order(dma_order(:,5)==1,[2 1]);

% Create variable to facilitate computations later on
match_dmas=zeros(length(actual_dma_groups(:,1)),1);
useme=dma_order(dma_order(:,5)==1,[2 1]);
for k=1:length(actual_dma_groups(:,1))
   test2=find(useme(:,1)==ones(length(predicted_dma_groups(:,1)),1)*actual_dma_groups(k,1) & useme(:,2)==ones(length(predicted_dma_groups(:,1)),1)*actual_dma_groups(k,2)); 
   
      if (isempty(test2)==1)
      else
          % for each observation in Nielsen, which observation of the
          % shortened to uniques only dma_order corresponds
          match_dmas(k)=test2;
   
      end
   

end

actual_dma_ratings=grpstats(nielsen_dma_ratings,nielsen_dmas_years)./100;
mean_demographics=demographics(dma_order(:,5)==1,:);


num_of_household_draws=2500;
% [mediamark_household_data mediamark_household_data_max hh_demographics mediamark_has_cable_or_satellite wave]=create_households_from_individual_data(raw_mediamark,num_of_household_draws);
% save mediamark_hh_data mediamark_household_data mediamark_household_data_max hh_demographics mediamark_has_cable_or_satellite wave
load mediamark_hh_data

mediamark_household_hours_watched=zeros(num_of_household_draws,sum(specification(:,3)));
mediamark_household_hours_watched_max=zeros(num_of_household_draws,sum(specification(:,3)));
mediamark_any_hours_watched=zeros(num_of_household_draws,sum(specification(:,3)));
k=1;
for j=1:375
    if (specification(j,3)==1)
        mediamark_household_hours_watched(:,k)=mediamark_household_data(:,j);
        mediamark_household_hours_watched_max(:,k)=mediamark_household_data_max(:,j);
        mediamark_any_hours_watched(:,k)=(mediamark_household_data(:,j)>0);
        k=k+1;
    end
end

%has_nielsen=(max(nielsen_raw_data(:,4:378))~=0)';
%has_mediamark=(max(mediamark_household_data~=0))';
% Compute mediamark moments
% Demographic order in estimation: [family income black asian hispanic college age];


mean_viewership_by_demographic=zeros(10,number_of_channels);

% Mean viewership by demographic group
    
    % family
    mean_viewership_by_demographic(1,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,6)>0,:));
        
    % income under 40k
    mean_viewership_by_demographic(2,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,2)<40,:));
    
    % income over 40k
    mean_viewership_by_demographic(3,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,2)>=40,:));
    
    % black
    mean_viewership_by_demographic(4,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,7)==2,:));

    % asian
    mean_viewership_by_demographic(5,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,7)==4,:));

    % hispanic
    mean_viewership_by_demographic(6,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,7)==3,:));
    
    % college 
    mean_viewership_by_demographic(7,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,8)==1,:));
    
    % age below 30
    mean_viewership_by_demographic(8,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,1)<30,:));    

    % age between 30 and 55
    mean_viewership_by_demographic(9,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,1)>=30 & hh_demographics(:,1)<=55,:));    

    % age over 55
    mean_viewership_by_demographic(10,:)= mean(mediamark_household_hours_watched_max(hh_demographics(:,1)>55,:));    
  
    % Cume by year
    hh_cume=mean(mediamark_any_hours_watched);  
    
    % Demographic cumes (Family=1, Black=1, Over 55=1)
    demographic_cume(1,:)=mean(mediamark_any_hours_watched(hh_demographics(:,6)>0 & hh_demographics(:,7)==2 & hh_demographics(:,1)>=55,:));
    demographic_cume(2,:)=mean(mediamark_any_hours_watched(hh_demographics(:,6)>0 & hh_demographics(:,7)~=2 & hh_demographics(:,1)>=55,:));
    demographic_cume(3,:)=mean(mediamark_any_hours_watched(hh_demographics(:,6)>0 & hh_demographics(:,7)~=2 & hh_demographics(:,1)<55,:));
    demographic_cume(4,:)=mean(mediamark_any_hours_watched(hh_demographics(:,6)>0 & hh_demographics(:,7)==2 & hh_demographics(:,1)<55,:));
    demographic_cume(5,:)=mean(mediamark_any_hours_watched(hh_demographics(:,6)==0 & hh_demographics(:,7)==2 & hh_demographics(:,1)>=55,:));
    demographic_cume(6,:)=mean(mediamark_any_hours_watched(hh_demographics(:,6)==0 & hh_demographics(:,7)~=2 & hh_demographics(:,1)>=55,:));
    demographic_cume(7,:)=mean(mediamark_any_hours_watched(hh_demographics(:,6)==0 & hh_demographics(:,7)~=2 & hh_demographics(:,1)<55,:));
    demographic_cume(8,:)=mean(mediamark_any_hours_watched(hh_demographics(:,6)==0 & hh_demographics(:,7)==2 & hh_demographics(:,1)<55,:));
    
    % Correlation in channel hours watched
    correlation_in_hh_viewership=corr(mediamark_household_hours_watched_max,mediamark_household_hours_watched_max);
    covariance_in_hh_viewership=cov(mediamark_household_hours_watched_max);
    mediamark_income_moments=[mean(mediamark_has_cable_or_satellite(hh_demographics(:,2)<25)) mean(mediamark_has_cable_or_satellite(hh_demographics(:,2)>=25 & hh_demographics(:,2)<45)) mean(mediamark_has_cable_or_satellite(hh_demographics(:,2)>=45 & hh_demographics(:,2)<70)) mean(mediamark_has_cable_or_satellite(hh_demographics(:,2)>=70 & hh_demographics(:,2)<100)) mean(mediamark_has_cable_or_satellite(hh_demographics(:,2)>=100 & hh_demographics(:,2)<150)) mean(mediamark_has_cable_or_satellite(hh_demographics(:,2)>=150))];

    ratings_data={nielsen_dma_ratings,universe_cume,coverage_cume,mean_viewership_by_demographic,demographic_cume,correlation_in_hh_viewership,hh_cume,covariance_in_hh_viewership};

%% END IMPORT VIEWERSHIP DATA

%% CREATE UTILITY MATRICES FROM DATA FOR COMPUTATIONAL EASE

% % % % % % % Create a useful sparse matrix that for each good (row), has a 1 in the
% % % % % % % columns of the other goods that are in its market
% % % % % % % It takes a long time to construct these matrices, so do it
% % % % % % % only the first time you load a new data set. 
same_market=sparse(length(dma_order(:,1)),length(dma_order(:,1)));
same_dma=sparse(length(dma_order(:,1)),length(dma_order(:,1)));
marked_market=zeros(11,210,110);
market_representative=zeros(length(dma_order(:,1)),1);
for i=1:length(dma_order(:,1))
    for j=1:length(dma_order(:,1))
        
        if (dma_order(i,1)==dma_order(j,1) && dma_order(i,2)==dma_order(j,2))
            same_dma(i,j)=1;
            same_dma(j,i)=1;
            if (dma_order(i,3)==dma_order(j,3))
            
                
                if (marked_market(dma_order(i,1)-1996,dma_order(i,2),dma_order(i,3))~=1) 
                    market_representative(i)=1;
                    marked_market(dma_order(i,1)-1996,dma_order(i,2),dma_order(i,3))=1;
                end
            same_market(i,j)=1;
            same_market(j,i)=1;
            
            end
            
        end
    end
end

same_market_min_max_index=zeros(length(dma_order(:,1)),2);
same_dma_min_max_index=zeros(length(dma_order(:,1)),2);
for i=1:length(dma_order(:,1))
    
    same_market_min_max_index(i,1)=find(same_market(i,:)==1,1);
    same_market_min_max_index(i,2)=find(same_market(i,:)==1,1,'last');
    
    same_dma_min_max_index(i,1)=find(same_dma(i,:)==1,1);
    same_dma_min_max_index(i,2)=find(same_dma(i,:)==1,1,'last');

end

save same_market same_dma same_market market_representative same_market_min_max_index same_dma_min_max_index

% A useful matrix for aggregating the satellite share
load same_market;
sat_dma_rep=same_dma*spdiags(market_representative.*system_relative_dma_size,0,length(market_representative),length(market_representative));
sys_size_vector=(dma_order(:,8)==1).*system_relative_dma_size;
system_share=same_market*bundle_market_shares;
save sat_dma_rep sat_dma_rep system_share sys_size_vector


% Initial values of the deltas in exponential form (must be > 0).
 last_exponential_deltas=ones(size([bundle_prices bundle_prices]));
save last_exponential_deltas last_exponential_deltas

% Iteration is used later to decide whether to create or load the GMM
% weighting matrix.
iteration=1;



%% END CREATE UTILITY MATRICES

disp('Data Imported.')
