#include <fintrf.h>
!
!     Matlab call :  [d_dma_time_watched d_hh_viewing]=dma_stats(time_watched_cable,time_watched_satellite,d_time_watched_cable,d_time_watched_satellite,d_bundle_share,d_satellite_share,bundle_shares_by_household,satellite_shares_by_household,same_dma_min_max_index,same_market_min_max_index,sys_size_vector,system_relative_dma_size,number_of_channels_vector);
!
!       time_watched_cable                            number_of_bundles x simulations x number_of_channels
!       time_watched_satellite                        number_of_bundles x simulations x number_of_channels
!       d_time_watched_cable                          number_of_channels x simulations x number_of_bundles
!       d_time_watched_satellite                      number_of_channels x simulations x number_of_bundles
!       d_bundle_share                                number_of_bundles x simulations
!       d_satellite_share                             number_of_bundles x simulations          
!       bundle_shares_by_household                    number_of_bundles x simulations
!       satellite_shares_by_household                 number_of_bundles x simulations  
!       same_dma_min_max_index                        number_of_bundles x 2
!       same_market_min_max_index                     number_of_bundles x 2
!       sys_size_vector                               number_of_bundles x 1
!       system_relative_dma_size                      number_of_bundles x 1
!       number_of_channels_vector                     number_of_channels x 1                           

!       Output
!       d_dma_time_watched                          number_of_bundles x number_of_channels
!       d_hh_viewing                                number_of_bundles x simulations x number_of_channels

!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer d_time_watched_cable, d_time_watched_satellite, time_watched_cable, time_watched_satellite, d_bundle_share, d_satellite_share, bundle_shares_by_household, satellite_shares_by_household, same_dma_min_max_index, same_market_min_max_index, sys_size_vector, system_relative_dma_size, number_of_channels_vector
      mwpointer d_dma_time_watched, d_hh_viewing
      mwsize mxgetm, mxgetn
      mwsize number_of_channels, number_of_bundles, simulations
      integer time_watched_dimensions(3)

!     RHS parameters

       time_watched_cable=mxgetpr(prhs(1)) 
       time_watched_satellite=mxgetpr(prhs(2)) 
       d_time_watched_cable=mxgetpr(prhs(3))  
       d_time_watched_satellite=mxgetpr(prhs(4)) 
       d_bundle_share=mxgetpr(prhs(5)) 
       d_satellite_share=mxgetpr(prhs(6)) 
       bundle_shares_by_household=mxgetpr(prhs(7)) 
       satellite_shares_by_household=mxgetpr(prhs(8)) 
       same_dma_min_max_index=mxgetpr(prhs(9)) 
       same_market_min_max_index=mxgetpr(prhs(10)) 
       sys_size_vector=mxgetpr(prhs(11)) 
       system_relative_dma_size=mxgetpr(prhs(12)) 
       number_of_channels_vector=mxgetpr(prhs(13))
 
        simulations=mxgetn(prhs(6))
        number_of_channels=mxgetm(prhs(13))      
        number_of_bundles=mxgetm(prhs(6))
  
        time_watched_dimensions(1)=number_of_bundles
        time_watched_dimensions(2)=simulations        
        time_watched_dimensions(3)=number_of_channels
        
      PLHS(1) = MXCREATEDOUBLEMATRIX(number_of_bundles,number_of_channels,0)
      d_dma_time_watched = MXGETPR(PLHS(1))

      PLHS(2) = mxCreateNumericArray(3,time_watched_dimensions, mxClassIDFromClassName('double'), 0)
      d_hh_viewing = MXGETPR(PLHS(2))

      call dma_stats(%val(d_dma_time_watched),%val(d_hh_viewing),%val(d_time_watched_cable),%val(d_time_watched_satellite),%val(time_watched_cable),%val(time_watched_satellite),%val(d_bundle_share),%val(d_satellite_share),%val(bundle_shares_by_household),%val(satellite_shares_by_household),%val(same_dma_min_max_index),%val(same_market_min_max_index),%val(sys_size_vector),%val(system_relative_dma_size),number_of_bundles,simulations,number_of_channels)
                               
      return
      end


        subroutine dma_stats(d_dma_time_watched,d_hh_viewing,d_time_watched_cable,d_time_watched_satellite,time_watched_cable,time_watched_satellite,d_bundle_share,d_satellite_share,bundle_shares_by_household,satellite_shares_by_household,same_dma_min_max_index,same_market_min_max_index,sys_size_vector,system_relative_dma_size,number_of_bundles,simulations,number_of_channels)
        implicit none
        mwsize number_of_channels, simulations, number_of_bundles
        integer i, j, k, m, n
        double precision d_dma_time_watched(number_of_bundles,number_of_channels), d_hh_viewing(number_of_bundles,simulations,number_of_channels)
        double precision d_time_watched_cable(number_of_channels,simulations,number_of_bundles), d_time_watched_satellite(number_of_channels,simulations,number_of_bundles), time_watched_cable(number_of_bundles,simulations,number_of_channels), time_watched_satellite(number_of_bundles,simulations,number_of_channels)
        double precision d_bundle_share(number_of_bundles,simulations), d_satellite_share(number_of_bundles,simulations), bundle_shares_by_household(number_of_bundles,simulations), satellite_shares_by_household(number_of_bundles,simulations)
        double precision same_dma_min_max_index(number_of_bundles,2), same_market_min_max_index(number_of_bundles,2), sys_size_vector(number_of_bundles), system_relative_dma_size(number_of_bundles)
        double precision cable_use(number_of_bundles,simulations,number_of_channels), satellite_use(number_of_bundles,simulations,number_of_channels), mean_cable(number_of_bundles,number_of_channels), mean_satellite(number_of_bundles,number_of_channels), cable_system_viewing(number_of_bundles,number_of_channels), dma_cable(number_of_bundles,number_of_channels)
        integer i_same_dma_min_max_index(number_of_bundles,2), i_same_market_min_max_index(number_of_bundles,2)
        integer*4, external :: mexPrintf
        character(len=80) :: line        
        
        i_same_dma_min_max_index=INT(same_dma_min_max_index)
        i_same_market_min_max_index=INT(same_market_min_max_index)
                
    !write(line,*) "tol = ", d_time_watched_satellite(1,3,1)
    !n = mexPrintf(line//achar(10)) 

  do k=1,number_of_channels
  do i=1,number_of_bundles
       
  do j=1,simulations


cable_use(i,j,k)=d_time_watched_cable(k,j,i)*bundle_shares_by_household(i,j) + d_bundle_share(i,j)*time_watched_cable(i,j,k)

satellite_use(i,j,k)=d_time_watched_satellite(k,j,i)*satellite_shares_by_household(i,j) + d_satellite_share(i,j)*time_watched_satellite(i,j,k)

d_hh_viewing(i,j,k) =cable_use(i,j,k) + satellite_use(i,j,k)

  end do

mean_cable(i,k)=SUM(cable_use(i,:,k))/simulations
mean_satellite(i,k)=SUM(satellite_use(i,:,k))/simulations

end do
end do



  !do i=1,number_of_bundles
!mean_cable(i,k)=SUM(cable_use(i,:,k))/simulations
!mean_satellite(i,k)=SUM(satellite_use(i,:,k))/simulations
    !end do

do m=1,number_of_channels

do n=1,number_of_bundles
cable_system_viewing(n,m)=SUM(mean_cable(i_same_market_min_max_index(n,1):i_same_market_min_max_index(n,2),m))*system_relative_dma_size(n)
end do

do n=1,number_of_bundles
dma_cable(n,m)=SUM(cable_system_viewing(i_same_dma_min_max_index(n,1):i_same_dma_min_max_index(n,2),m))

d_dma_time_watched(n,m)=dma_cable(n,m) + DOT_PRODUCT(sys_size_vector(i_same_dma_min_max_index(n,1):i_same_dma_min_max_index(n,2)),mean_satellite(i_same_dma_min_max_index(n,1):i_same_dma_min_max_index(n,2),m));  
   
    end do

end do



        return
        end

